<#--########################################################################
  Warning! Do not delete this header, it is used by StrategyQuant to get
  information about this code generator.

  <name>Expert Advisor for MetaTrader4 (*.MQ4)</name>
  <id>mt4</id>
  <extensionName>MetaTrader4 Expert Advisor</extensionName>
  <extension>mq4</extension>
  <description>&lt;html&gt;Saves strategy as MetaTrader4 EA.&lt;br/&gt;
  You should save this file to {MetaTrader Data Folder}&lt;strong&gt;/MQL4/Experts&lt;/strong&gt; directory.&lt;/html&gt;</description>
#########################################################################-->
<@compress single_line=true>
<#include "./global/globalFunctions.inc">
<#include "./MetaTrader4/Functions.inc">
<#include "./MetaTrader4/mt4Parameters.inc">
<#include "./MetaTrader4/mt4Size.inc">
<#include "./MetaTrader4/mt4TradingOptions.inc">
<#include "./MetaTrader4/mt4Blocks.inc">
</@compress>
<#if getRulesCountForEvent("OnBarUpdate") == 0 >
 Strategy doesn't contain any complete rule, there's no EA code to generate.
<#else>
//+------------------------------------------------------------------+
//| ${getOptions("StrategyName")} EA
//|
//| Generated by StrategyQuant ${getOptions("Version")} at ${getOptions("Date")}
//|
//| This EA uses 3rd party library OrderReliable for industry-strength 
//| order handling. OrderReliable library is normally distributed under 
//| GPL license. Its author provided it under MIT license limited to 
//| usage only within strategies generated by StrategyQuant.  
//|   
//| This means that this library doesn't restrict you to use your EA 
//| generated by SQ comercially in any way you want.  
//| But if you'll use any method from OrderReliable library outside 
//| of this EA in your own EAs or projects, its use will be governed 
//| by its standard GPL license.  
//| 
//| Scroll to the very bottom of this source code for the full text 
//| of OrderReliable license. 
//+------------------------------------------------------------------+

#property copyright "StrategyQuant.com"
#property link      "http://www.StrategyQuant.com"

#include <stdlib.mqh>
#include <stderror.mqh>

const int SLPTTYPE_RANGE = 0;
const int SLPTTYPE_LEVEL = 1;

//+------------------------------------------------------------------+
// -- Variables
//+------------------------------------------------------------------+
extern string CustomComment = "${getOptions("StrategyName")}";

<#if hasVariables() >
<@mt4PrintVariables />
</#if>

<@mt4PrintSubcharts />

//+------------------------------------------------------------------+
// OrderReliable library variables
//+------------------------------------------------------------------+
string 	OrderReliableVersion = "v36";
string 	OrderReliable_Fname = "OrderReliable (Function Name Not Set)";

int 	retry_attempts 		      = 10000; // general retry attempts
int 	retry_attempts_bad_price   = 3;  // retry attempts if the error is bad price
double 	sleep_time 			      = 2.0;
double 	sleep_maximum 		      = 20.0;


int 	ErrorLevel 			= 3;

bool	UseLimitToMarket 	= false;
bool	UseForTesting 		= false;
bool	AddSpreadToComment	= false;

//+------------------------------------------------------------------+
// -- SQ internal variables
// add word "extern" in front of the variable you want
// to make configurable
//+------------------------------------------------------------------+
int sqMaxEntrySlippage = 5;          //Max tolerated entry slippage in pips. Zero means unlimited slippage
int sqMaxCloseSlippage = 0;          //Max tolerated close slippage in pips. Zero means unlimited slippage       
bool autoCorrectMaxSlippage = true;  //If set to true, it will automatically adjust max slippage according to symbol digits (*10 for 3 and 5 digit symbols)  

double sqMinDistance = ${getOptions("MinDistance")};

//+------------------------------------------------------------------+
// Verbose mode values:
// 0 - don't print messages to log at all
// 1 - print messages to log only when trading (not when backtesting)
// 2 - print messages to log always
//+------------------------------------------------------------------+
int sqVerboseMode = 2;

extern bool sqDisplayInfoPanel = true;
extern bool ModifyInsteadOfReplacing = true;

extern int OpenBarDelay = 0; // open bar delay in minutes
// it can be used for Daily strategies to trigger trading a few minutes later -
// because brokers sometimes have technical delay after midnight and we have to postpone order execution

int sqLabelCorner = 1;
int sqOffsetHorizontal = 5;
int sqOffsetVertical = 20;
color sqLabelColor = White;

datetime _sqLastOpenBarTime = 0;
bool _sqIsBarOpen = false;
double gPointCoef = 0;
int _ticket;                     
bool cond[100];

double initialBalance = 0;                    
    
bool openingOrdersAllowed = true;
bool firstCall = true;

<#assign ExitMethodsList = ["StopLoss", "ProfitTarget", "MoveSL2BE", "TrailingStop", "ExitAfterBars"] />
<@getAllMagicNumbers />

//+------------------------------------------------------------------+
// -- Functions
//+------------------------------------------------------------------+

void OnTick() {

   sqInitStart();       
   
   <@printManageOrdersForMagicNumbers />
   
   openingOrdersAllowed = sqHandleTradingOptions();

<#if getRulesCountForEvent("OnBarUpdate") gt 0 >
<@printRulesForEvent "OnBarUpdate" />
</#if>
   return;
}

//+------------------------------------------------------------------+

int OnInit() {
   VerboseLog("--------------------------------------------------------");
   VerboseLog("Starting the EA");

   gPointCoef = calculatePointCoef(Symbol());

   VerboseLog("--------------------------------------------------------");

   if(sqDisplayInfoPanel) {
     sqInitInfoPanel();
   }

   SQTime = new CSQTime();

   if(!IsTesting() && !IsOptimization()) {                                         
      initTimer();
   }
           
   initialBalance = AccountInfoDouble(ACCOUNT_BALANCE);
   
   <@printTradingOptionsInit />
   
   <@correctSubchartSymbols />
   
<#if getRulesCountForEvent("OnInit") gt 0 >
   int _orderType, _ticket, _temp;

   // OnInit  rules
<@printRulesForEvent "OnInit" />
</#if>

   double minDistanceMT = NormalizeDouble(MarketInfo(Symbol(), MODE_STOPLEVEL) * MarketInfo(Symbol(), MODE_POINT), Digits);
   double minDistanceSQ = NormalizeDouble(sqMinDistance * sqGetPointCoef(Symbol()), Digits);
   
   if(minDistanceSQ < minDistanceMT){
      VerboseLog("--------------------------------------------------------");
      VerboseLog("Warning! Min distance of this symbol is greater than min distance set in SQ! The backtest results may differ");
      VerboseLog("MT min distance: ", DoubleToStr(minDistanceMT), ", SQ min distance: ", DoubleToStr(minDistanceSQ));
      VerboseLog("--------------------------------------------------------");
   }

   return(0);
}

//+------------------------------------------------------------------+

void OnDeinit(const int reason) {
   sqDeinitInfoPanel();
   
<#if getRulesCountForEvent("OnDeinit") gt 0 >
   // OnDenit  rules
<@printRulesForEvent "OnDeinit" />
</#if>
   delete SQTime;
   <@printTradingOptionsDeinit />
   
   return;
}
    
//+------------------------------------------------------------------+

void initTimer(){
   int period = 20 * 3600;                //20 hours
   period += MathRand() % (10 * 3600);    //add another 0-10 hours randomly

   if(!EventSetTimer(period)){
      VerboseLog("Cannot set timer. Error code: ", GetLastError());
   }
}

//+------------------------------------------------------------------+

void OnTimer(){
   //clear unused variables
   int deletedCount = 0;
   
   VerboseLog("Clearing variables...");
   
   for(int a=GlobalVariablesTotal() - 1; a>=0; a--){
      string variableName = GlobalVariableName(a);

      if(GlobalVariableCheck(variableName)){
         string variableNameParts[];
         int parts = StringSplit(variableName, '_', variableNameParts);
         
         if(parts != 3 || variableNameParts[0] != getVariablePrefix()) continue;
         
         int ticketNo = StrToInteger(variableNameParts[1]);
         
         bool variableUsed = false;
         
         for(int i=0; i < OrdersTotal(); i++) {
            if(OrderSelect(i, SELECT_BY_POS) == true) {
               if(OrderTicket() == ticketNo){
                  variableUsed = true;
                  break;
               }
            }
         }
         
         ResetLastError();
         
         if(!variableUsed){
            if(GlobalVariableDel(variableName)){
               deletedCount++;
            }
            else {
               VerboseLog("Cannot delete variable. Error code: ", GetLastError());
            }
         }
      }
   }
   
   VerboseLog(IntegerToString(deletedCount), " variables cleared");
}

//+------------------------------------------------------------------+

bool sqHandleTradingOptions() {
<@printTradingOptionsHandlers />
}   

//+------------------------------------------------------------------+

bool checkMagicNumber(int magicNo){
<#list MagicNumbersList as magicNumber>
    if(magicNo == ${magicNumber}){
         return true;
    }
</#list>
    return false;
}

//+------------------------------------------------------------------+

double sqGetMarketTickSize(string symbol){
   symbol = correctSymbol(symbol);
    
   <@printMarketDigitsCases />
   
   return -1;
}

//+------------------------------------------------------------------+

double sqGetGlobalSL(string symbol, int orderType, double price) {
   <#if doc.StrategyFile.Strategy.GlobalSLPT.useSameSLPTforBothDirections=="true">
   <@printPTSLValue "globalSL" "SL" />
   <#else>
   if(orderType == OP_BUY || orderType == OP_BUYLIMIT || orderType == OP_BUYSTOP) {
      <@printPTSLValue "globalLongSL" "SL" />
   } else {
      <@printPTSLValue "globalShortSL" "SL" />
   }
   </#if>
}

//+------------------------------------------------------------------+

double sqGetGlobalPT(string symbol, int orderType, double price) {
   <#if doc.StrategyFile.Strategy.GlobalSLPT.useSameSLPTforBothDirections=="true">
   <@printPTSLValue "globalPT" "PT" />
   <#else>
   if(orderType == OP_BUY || orderType == OP_BUYLIMIT || orderType == OP_BUYSTOP) {
      <@printPTSLValue "globalLongPT" "PT" />
   } else {
      <@printPTSLValue "globalShortPT" "PT" />
   }
   </#if>
}

//+------------------------------------------------------------------+

double sqGetIndicatorByIdentification(string idHash, int shift) {
   <@printIndicatorsUsedInFunctions  />

   return(0);
}


//+------------------------------------------------------------------+

double sqGetValueByIdentification(string idHash) {
   <@printValuesUsedInExitMethods />

   return(0);
}

//----------------------------------------------------------------------------

void sqManageOrders(int magicNumber) {
   if(_sqIsBarOpen) {
      for(int i=OrdersTotal()-1; i>=0; i--) {
         if (OrderSelect(i,SELECT_BY_POS)==true) {
            if(OrderMagicNumber() != magicNumber) {
               continue;
            }  
               
            int ticket = OrderTicket();

            if(OrderType() == OP_BUY || OrderType() == OP_SELL) {
               // handle Exit Methods only for live orders
               <#list ExitMethodsList as exitMethod>
                  <@printExitMethodManage exitMethod />
               </#list>
               
            }

            sqManageOrderExpiration(ticket);
         }

         if(OrdersTotal() <= 0) return;
      }
   }
}

//----------------------------------------------------------------------------

void sqResetGlobalVariablesForTicket(int ticket) {
   sqSetGlobalVariable(ticket, "sqOrderExpiration", 0);
   sqSetGlobalVariable(ticket, "sqOrderOpenTime", Time[0]);

  <#list ExitMethodsList as exitMethod>
    <@printExitMethodReset exitMethod />
  </#list>
  
}

<#include "./MetaTrader4/SQ.mqh">

<#include "./MetaTrader4/mt4SQIndicators.inc">

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
// ExitMethods includes
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+

<#list ExitMethodsList as exitMethod>
   <@printExitMethodInclude exitMethod />
</#list>

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
// Trading Options includes
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+

class CTradingOption {
public:
   virtual bool onBarUpdate() = 0;
};

//+------------------------------------------------------------------+

<@printTradingOptionsClasses />

<@printMoneyManagementClass />

<#include "./MetaTrader4/OrderReliable.mqh">

<#list includeFiles as includeFile>

//+----------------------------- Include from ${includeFile} -------------------------------------+

   <#include includeFile>
</#list>

</#if> <#-- strategy has no rules -->