<#--########################################################################
  Warning! Do not delete this header, it is used by StrategyQuant to get
  information about this code generator.

  <name>Expert Advisor for MetaTrader5 (*.MQ5)</name>
  <id>mt5</id>
  <extensionName>MetaTrader5 Expert Advisor</extensionName>
  <extension>mq5</extension>
  <description>&lt;html&gt;Saves strategy as MetaTrader5 EA.&lt;br/&gt;
  You should save this file to {MetaTrader Data Folder}&lt;strong&gt;/MQL5/Experts&lt;/strong&gt; directory.&lt;/html&gt;</description>
#########################################################################-->
<@compress single_line=true>
<#include "./global/globalFunctions.inc">
<#include "./MetaTrader5/Functions.inc">
<#include "./MetaTrader5/mt5Parameters.inc">
<#include "./MetaTrader5/mt5Size.inc">
<#include "./MetaTrader5/mt5Blocks.inc">     
<#include "./MetaTrader5/mt5TradingOptions.inc">
</@compress>
<#if getRulesCountForEvent("OnBarUpdate") == 0 >
 Strategy doesn't contain any complete rule, there's no EA code to generate.
<#else>
//+------------------------------------------------------------------+
//| ${getOptions("StrategyName")} EA
//|
//|    Generated by StrategyQuant ${getOptions("Version")} for ${getOptions("Engine")}
//|    Generated at ${getOptions("Date")}
//+------------------------------------------------------------------+

#property copyright "StrategyQuant.com"
#property link      "http://www.StrategyQuant.com"    

#include <Expert/Expert.mqh>
#include <Trade/SymbolInfo.mqh>
#include <Trade/OrderInfo.mqh>
#include <Trade/HistoryOrderInfo.mqh>
#include <Trade/PositionInfo.mqh>
#include <Trade/DealInfo.mqh>
#include <Trade/TerminalInfo.mqh>    
#include <Object.mqh>
#include <MovingAverages.mqh>

const int SLPTTYPE_RANGE = 0;
const int SLPTTYPE_LEVEL = 1;

const ENUM_ORDER_TYPE_FILLING preferredFillingType = ORDER_FILLING_FOK;         //preferred filling type - will be applied if available
const bool forceFillingType = false;                                            //if set to true, it will force using preferredFillingType when opening orders

//+------------------------------------------------------------------+
// -- Variables
//+------------------------------------------------------------------+
input string CustomComment = "${getOptions("StrategyName")}";

<#if hasVariables() >
<@mt5PrintVariables />
</#if>

<@mt5PrintSubcharts />

//+------------------------------------------------------------------+
// -- SQ Variables
// - add word "input" in front of the variable you want
//   to make configurable
//+------------------------------------------------------------------+
                                                                               
int sqMaxEntrySlippage = 5;          //Max tolerated entry slippage in pips. Zero means unlimited slippage
int sqMaxCloseSlippage = 0;          //Max tolerated close slippage in pips. Zero means unlimited slippage       
bool autoCorrectMaxSlippage = true;  //If set to true, it will automatically adjust max slippage according to symbol digits (*10 for 3 and 5 digit symbols)  

double sqMinDistance = ${getOptions("MinDistance")}; //Stop orders min distance from current price

//+------------------------------------------------------------------+
// Verbose mode values:
// 0 - don't print messages to log at all
// 1 - print messages to terminal log 
// 2 - print messages to file
//+------------------------------------------------------------------+      

int sqVerboseMode = 1;

input int OpenBarDelay = 0; //Open bar delay in minutes
// it can be used for Daily strategies to trigger trading a few minutes later -
// because brokers sometimes have technical delay after midnight and we have to postpone order execution

input string slex = "----------- Order expiration time (for stocks) -----------";
input int ExpirationTime = 0; //Order expiration time in minutes    

int magicNumber;
int minBars = 30;
int sqMaxRetries = 5;           
bool openingOrdersAllowed = true;      
        
bool sqDisplayInfoPanel = MQLInfoInteger(MQL_TESTER) == 0 && MQLInfoInteger(MQL_OPTIMIZATION) == 0;
int sqLabelCorner = 1;
int sqOffsetHorizontal = 5;
int sqOffsetVertical = 20;
color sqLabelColor = clrWhite;

<#assign ExitMethodsList = ["StopLoss", "ProfitTarget", "MoveSL2BE", "TrailingStop", "ExitAfterBars"] />
<@getAllMagicNumbers />

int indicatorHandles[];

//Indicator buffer indexes definitions
<@printIndicatorConstants />

struct LastClosedTrade {
   string symbol;
   int magicNumber;
   datetime closeTime;
};

LastClosedTrade lastClosedTrades[];

MqlTradeRequest mrequest;  // Used for sending our trade requests
MqlTradeResult mresult;    // Used to get our trade results
MqlRates mrate[];          // Used to store the prices, volumes and spread of each bar
CTerminalInfo terminalInfo;
                       
string sqStrategyName = "${getOptions("StrategyName")}";
datetime startTime = TimeCurrent();
datetime lastCheckedTradeTime = startTime;
                
double gPointCoef = 0;                  

int sqTicket = 1;
datetime lastBarTime;
bool _sqIsBarOpen;
bool cond[100];

double initialBalance = 0;

string valueIdentificationSymbol = "";      
string StrategyID = "${getOptions("ID")}";

bool firstCall = true;     
bool timerInitialized = false;

//+------------------------------------------------------------------+
// -- Functions
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick() {
   //--- Do we have enough bars to work with?
   if(Bars(_Symbol,_Period) < minBars) {   // if total bars is less than minBars
      Alert(StringFormat("NOT ENOUGH DATA: Less Bars than %d", minBars));
      return;
   }

   //--- Get the details of the latest 2 bars
   if(CopyRates(_Symbol, _Period, 0, 2, mrate) < 2) {
      Alert("Error copying rates/history data - error:", GetLastError(), "!!");
      ResetLastError();
      return;
   }
     
   ZeroMemory(mrequest);      // Initialization of mrequest structure
 
   if(!timerInitialized) initTimer();

   checkBarOpen();
   
   if(sqDisplayInfoPanel) {
      sqTextFillOpens();
      if(_sqIsBarOpen) {
         sqTextFillTotals();   
      }  
   }
    
	 <#list MagicNumbersList as magicNumber>
   sqManagePositions(${magicNumber});
	 </#list>    
   
   openingOrdersAllowed = sqHandleTradingOptions();
   
   ulong _ticket;                    
   double openPrice;  
   double sl, pt;
   
<#if getRulesCountForEvent("OnBarUpdate") gt 0 >
<@printRulesForEvent "OnBarUpdate" />
</#if>

}      

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit(){
   VerboseLog("--------------------------------------------------------");
   VerboseLog("Starting the EA");

   //initMagicNumber();
   if(!initIndicators()) return(INIT_FAILED);
   
   gPointCoef = calculatePointCoef(Symbol());

   //VerboseLog("Broker stop level: ", SymbolInfoInteger(_Symbol, SYMBOL_TRADE_STOPS_LEVEL));

   VerboseLog("--------------------------------------------------------");          
   
   if(sqDisplayInfoPanel) {
     sqInitInfoPanel();
   }
   
   SQTime = new CSQTime();
    
   if(MQLInfoInteger(MQL_TESTER) == 0 && MQLInfoInteger(MQL_OPTIMIZATION) == 0){
      initTimer();   
   }         
   else {
      timerInitialized = true;
   }
   
   initialBalance = AccountInfoDouble(ACCOUNT_BALANCE);
   
   <@printTradingOptionsInit />
   
<#if getRulesCountForEvent("OnInit") gt 0 >
   int _orderType, _ticket, _temp;

   // OnInit  rules
<@printRulesForEvent "OnInit" />
</#if>
   
   double minDistanceMT = NormalizeDouble(SymbolInfoInteger(Symbol(), SYMBOL_TRADE_STOPS_LEVEL) * SymbolInfoDouble(Symbol(), SYMBOL_POINT), Digits());
   double minDistanceSQ = NormalizeDouble(sqMinDistance * sqGetPointCoef(Symbol()), Digits());
   
   if(minDistanceSQ < minDistanceMT){
      VerboseLog("--------------------------------------------------------");
      VerboseLog("Warning! Min distance of this symbol is greater than min distance set in SQ! The backtest results may differ");
      VerboseLog("MT min distance: ", DoubleToString(minDistanceMT), ", SQ min distance: ", DoubleToString(minDistanceSQ));
      VerboseLog("--------------------------------------------------------");
   }
   
   return(INIT_SUCCEEDED);
}

//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason) {
  
   sqDeinitInfoPanel();
  
<#if getRulesCountForEvent("OnDeinit") gt 0 >
 // OnDeinit  rules
<@printRulesForEvent "OnDeinit" />
</#if>
                                    
   writeReportFile();  
                              
   delete SQTime;
   <@printTradingOptionsDeinit />
   
}
         
//+------------------------------------------------------------------+

void initTimer(){
   int period = 20 * 3600;                //20 hours
   period += MathRand() % (10 * 3600);    //add another 0-10 hours randomly
   
   if(!EventSetTimer(period)){
      VerboseLog("Cannot set timer. Error code: ", IntegerToString(GetLastError()));
   }      
   else {
      timerInitialized = true;
   }
}

//+------------------------------------------------------------------+

void OnTimer(){
   //clear unused variables
   int deletedCount = 0;
   
   VerboseLog("Clearing variables...");
   
   for(int a=GlobalVariablesTotal() - 1; a>=0; a--){
      string variableName = GlobalVariableName(a);
      
      if(GlobalVariableCheck(variableName)){
         string variableNameParts[];
         int parts = StringSplit(variableName, '_', variableNameParts);
         
         if(parts != 3 || StringFind(variableNameParts[0], getVariablePrefix()) != 0) continue;
         
         long ticketNo = StringToInteger(variableNameParts[1]);

         bool variableUsed = false;
         
         if(PositionSelectByTicket(ticketNo)) {    // check if position with this ticket exists
            variableUsed = true;    
         }
         else {                                    // check if pending order with this ticket exists
            for (int i = 0; i < OrdersTotal(); i++) {
               if(OrderGetTicket(i) == ticketNo){
                  variableUsed = true;
                  break;
               }
            }
         }
         
         ResetLastError();
         
         if(!variableUsed){
            if(GlobalVariableDel(variableName)){
               deletedCount++;
            }
            else {
               VerboseLog("Cannot delete variable. Error code: ", IntegerToString(GetLastError()));
            }
         }
      }
   }
   
   VerboseLog(IntegerToString(deletedCount), " variables cleared");
}

//+------------------------------------------------------------------+

bool sqHandleTradingOptions() {
<@printTradingOptionsHandlers />
}

//+------------------------------------------------------------------+

bool checkMagicNumber(ulong magicNo){
<#list MagicNumbersList as magicNumber>
    if(magicNo == ${magicNumber}){
         return true;
    }
</#list>
    return false;
}

//+------------------------------------------------------------------+
//| Trade transaction event                                             |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+

void updateLastTrades(){
    HistorySelect(lastCheckedTradeTime, TimeCurrent());
    lastCheckedTradeTime = TimeCurrent();
    
    for(int i=HistoryDealsTotal() - 1; i>=0; i--) {
        ulong ticket = HistoryDealGetTicket(i);
        string dealSymbol = HistoryDealGetString(ticket, DEAL_SYMBOL);
        int dealMagic = (int) HistoryDealGetInteger(ticket, DEAL_MAGIC);
        datetime dealTime = (datetime) HistoryDealGetInteger(ticket, DEAL_TIME);
      
        if(HistoryDealGetInteger(ticket, DEAL_ENTRY) != DEAL_ENTRY_OUT) continue;
        
        int lastTradeIndex = getLastTradeIndex(dealSymbol, dealMagic);
        if(lastTradeIndex >= 0){
           if(dealTime > lastClosedTrades[lastTradeIndex].closeTime){
               lastClosedTrades[lastTradeIndex].closeTime = dealTime;
           }
        }
        else {
           int prevSize = ArraySize(lastClosedTrades);
           ArrayResize(lastClosedTrades, prevSize + 1, 10);
           
           lastClosedTrades[prevSize].symbol = dealSymbol;
           lastClosedTrades[prevSize].magicNumber = dealMagic;
           lastClosedTrades[prevSize].closeTime = dealTime;
        }
    }
}

//+------------------------------------------------------------------+

bool sqTradeRecentlyClosed(string symbol, int magicNo, bool checkThisBar, bool checkThisMinute) {         
    updateLastTrades();
    
    string tradeSymbol = correctSymbol(symbol);
    int lastTradeIndex = -1;
    
    if(tradeSymbol == "Any" || magicNo == 0){ 
        for(int a=0; a<ArraySize(lastClosedTrades); a++){
            if((tradeSymbol == "Any" || lastClosedTrades[a].symbol == tradeSymbol) && (magicNo == 0 || lastClosedTrades[a].magicNumber == magicNo)){
                if(lastTradeIndex < 0 || lastClosedTrades[a].closeTime > lastClosedTrades[lastTradeIndex].closeTime){
                    lastTradeIndex = a;
                }
            }
        }
    }
    else {
        lastTradeIndex = getLastTradeIndex(tradeSymbol, magicNo);
    }
    
    if(lastTradeIndex >= 0) {
        if(checkThisBar) {
           if(lastClosedTrades[lastTradeIndex].closeTime >= getTime(0)) {
              // order finished this bar
              return true;
           }
        }
  
        if(checkThisMinute) {
            string strCurrentsqTimeMinutes = TimeToString(TimeCurrent(), TIME_DATE | TIME_MINUTES);
            string strClosesqTimeMinutes = TimeToString(lastClosedTrades[lastTradeIndex].closeTime, TIME_DATE | TIME_MINUTES);
  
            if(strCurrentsqTimeMinutes == strClosesqTimeMinutes) {
                // order finished this minute
                return true;
            }
        }
    }
    
    return false;
}
   
//+------------------------------------------------------------------+

int getLastTradeIndex(string symbol, int magicNo){
    for(int a=0; a<ArraySize(lastClosedTrades); a++){
        if(lastClosedTrades[a].symbol == symbol && lastClosedTrades[a].magicNumber == magicNo) return a;
    }
    
    return -1;
}
   
//+------------------------------------------------------------------+

bool initIndicators(){
   
   <@printIndicatorsInitialization />
   
   for(int a=0; a<ArraySize(indicatorHandles); a++){
      //--- if the handle is not created 
      if(indicatorHandles[a] == INVALID_HANDLE) { 
         //--- tell about the failure and output the error code 
         Print("Failed to create handle of the indicator, error code %d", GetLastError()); 
         //--- the indicator is stopped early 
         return(false); 
      }
   }
   
   return(true);
}             

//+------------------------------------------------------------------+
 
bool isFibo(uchar indyIndex){
<@printFiboCases />
   return false;
}

//+------------------------------------------------------------------+

uchar getHeikenAshiIndex(string symbol, ENUM_TIMEFRAMES timeframe){
   symbol = correctSymbol(symbol);
   
   if(symbol == Symbol() && (timeframe == Period() || timeframe == 0)){
      return HEIKENASHI_1;
   }
   else {
<@printHeikenAshiCases />
   }

   return 255;
}

//+------------------------------------------------------------------+

double sqGetMarketTickSize(string symbol){
   if(!UseSQTickSize) return -1;
   symbol = correctSymbol(symbol);
    
   <@printMarketDigitsCases />
   
   return -1;
}

//+------------------------------------------------------------------+

double sqGetGlobalSL(string symbol, int orderType, double price) {
   <#if doc.StrategyFile.Strategy.GlobalSLPT.useSameSLPTforBothDirections=="true">
   <@printPTSLValue "globalSL" "SL" />
   <#else>
   if(orderType == ORDER_TYPE_BUY || orderType == ORDER_TYPE_BUY_LIMIT || orderType == ORDER_TYPE_BUY_STOP) {
      <@printPTSLValue "globalLongSL" "SL" />
   } else {
      <@printPTSLValue "globalShortSL" "SL" />
   }
   </#if>
}

//+------------------------------------------------------------------+

double sqGetGlobalPT(string symbol, int orderType, double price) {
   <#if doc.StrategyFile.Strategy.GlobalSLPT.useSameSLPTforBothDirections=="true">
   <@printPTSLValue "globalPT" "PT" />
   <#else>
   if(orderType == ORDER_TYPE_BUY || orderType == ORDER_TYPE_BUY_LIMIT || orderType == ORDER_TYPE_BUY_STOP) {
      <@printPTSLValue "globalLongPT" "PT" />
   } else {
      <@printPTSLValue "globalShortPT" "PT" />
   }
   </#if>
}

//+------------------------------------------------------------------+

double sqGetValueByIdentification(int idHash) {
   string symbol = valueIdentificationSymbol;
   
   <@printValuesUsedInExitMethods "#MoveSL2BE.MoveSL2BE#" "price1" />

   return(0);
}

//+------------------------------------------------------------------+

double sqGetExpressionByIdentification(string id, int shift) {
   <#list expressions as expression>
      if(id == "<@compress_single_line><@escape>${expression}</@escape></@compress_single_line>") {
          <@printModifiedExpression expression />;
      }
   </#list>
   
   return(0);
}

//+------------------------------------------------------------------+

bool sqIsExpression(string id) {
   <#list expressions as expression>
      if(id == "<@compress_single_line><@escape>${expression}</@escape></@compress_single_line>") {
          return(true);
      }
   </#list>
   
   return(false);
}
  
//+------------------------------------------------------------------+
    
void sqManagePositions(int magicNo) {
   if(_sqIsBarOpen){
     for (int cc = PositionsTotal() - 1; cc >= 0; cc--) {
        ulong positionTicket = PositionGetTicket(cc);
     
        if (PositionSelectByTicket(positionTicket)) {
           if(PositionGetInteger(POSITION_MAGIC) != magicNo) {      
              continue;
           }
           
           sqManageSL2BE(positionTicket);
           sqManageTrailingStop(positionTicket);
           sqManageExitAfterXBars(positionTicket);
        }
        
        if(PositionsTotal() <= 0) break;
     }
     
     sqManageOrderExpirations(magicNo);
   }
}                                                                                     

//+------------------------------------------------------------------+

<#include "./MetaTrader5/SQ.mqh">

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
// ExitMethods includes
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+

<#list ExitMethodsList as exitMethod>
   <@printExitMethodInclude exitMethod />
</#list>

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
// Trading Options includes
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+

class CTradingOption {
public:
   virtual bool onBarUpdate() = 0;
};

//+------------------------------------------------------------------+

<@printTradingOptionsClasses />


<@printMoneyManagementClass />

<#list includeFiles as includeFile>

//+----------------------------- Include from ${includeFile} -------------------------------------+

   <#include includeFile>
</#list>

</#if> <#-- strategy has no rules -->